from datetime import datetime
import logging

from osv import osv, fields

class inventory_processing(osv.osv_memory):
    _name = 'inventory.processing'
    _description = 'Inventory Processing'

    _columns = {'is_finished': fields.boolean('Is Finished'),
                'run_procurements': fields.boolean(
                    'Run Procurements',
                    help='Trigger all active procurements to see if they ' +
                        'are ready to complete.'),
                'minimum_stock_rules': fields.selection(
                    (('run', 'Run'),
                     ('auto', 'Run with automatic minimum')),
                    'Minimum Stock Rules',
                    help='Check all minimum stock rules and generate any ' +
                        'needed procurements.'),
                'run_sales_volumes': fields.boolean(
                    'Run Sales Volumes',
                    help='Update the sales history for all products, as ' +
                        'well as the daily and yearly sales and consumption.'),
                'sales_months': fields.integer(
                    'Sales months', 
                    help='Number of months of sales orders to include in ' +
                        'the calculation'), 
                'run_stockouts': fields.boolean(
                    'Run Stockout Simulation',
                    help='Calculate days until stockout and days until ' +
                        'reorder for all products, as well as likely ' +
                        'stockouts.'),
                'run_build_capacities': fields.boolean(
                    'Run Build Capacities',
                    help='Calculate how many of each product we can build ' +
                        'with current stock. Also update quantities in ' +
                        'outstanding procurements (purchase orders or slow ' +
                        'production orders) and already committed to ' +
                        'assemblies and finished products. These are ' +
                        'independent of the stockout simulation.'),
                'run_production_costs': fields.boolean(
                    'Run Production Costs',
                    help="Sum up the cost of each product's components."),
                'products_changed': fields.integer(
                    'Number of Products Updated', 
                    readonly=True)}
    
    _defaults = {'sales_months': lambda *a: 3}

    def run(self, cr, uid, ids, context=None):
        products_changed = 0
        for request in self.browse(cr, uid, ids):
            start_time = datetime.now()
            proc_obj = self.pool.get('procurement.order')
            prod_obj = self.pool.get('product.product')
            
            if request.run_procurements:
                proc_obj._procure_confirm(cr, 
                                          uid, 
                                          context=context)
            if request.minimum_stock_rules:
                automatic = request.minimum_stock_rules == 'auto'
                proc_obj._procure_orderpoint_confirm(cr, 
                                                     uid, 
                                                     automatic=automatic,
                                                     context=context)
            if request.run_sales_volumes:
                prod_obj._calculate_all_sales_volumes(
                    cr, 
                    uid, 
                    request.sales_months,
                    context)
            if request.run_stockouts:
                products_changed += prod_obj._calculate_all_stockouts(
                    cr, 
                    uid, 
                    context)
            if request.run_build_capacities:
                products_changed += prod_obj._calculate_all_build_capacities(
                    cr, 
                    uid, 
                    context)
            if request.run_production_costs:
                products_changed += prod_obj._calculate_all_production_costs(
                    cr, 
                    uid, 
                    context)
                
            duration = datetime.now() - start_time
            logger = logging.getLogger('inventory.processing')
            logger.info('Inventory processing took %ss.' % duration.seconds)
                
        self.write(cr, uid, ids, {'is_finished': True, 
                                  'products_changed': products_changed})
        return {} # no further action required
