import datetime
import time
from osv import fields
from osv import osv
from tools import config

class stock_location_path(osv.osv):
    _inherit = 'stock.location.path'

    def sanity_check(self, cr, uid, context=None):
        self.location_path_step_added(cr, uid, context)
        return 0

    def location_path_step_added(self, cr, uid, context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(
            cr,
            uid,
            context,
            "Found product paths that weren't Automatic No Step Added",
            "This is a stub to track incidents where products are configured "
            "to add more stock moves than necessary.",
            'stock_location_path_step_added')

        ids = self.search(cr,uid,[('auto', '!=', 'transparent')])
        if ids:
            problem_writeup = (
                "The following products have paths that aren't "
                "Automatic No Step Added:\n\n")
            for path in self.browse(cr,uid,ids,context):
                problem_writeup += "[%s] %s\n\n" % (
                    path.product_id.default_code,
                    path.product_id.name)

            # now that we've finished documenting the problem let's notify the 
            # appropriate users
            self.pool.get('crm.helpdesk').sanity_test_log(
                cr, uid, context,
                case_id,
                problem_writeup)


stock_location_path()

class stock_picking(osv.osv):
    _inherit = 'stock.picking'

    def sanity_check(self, cr, uid, context=None):
        self.po_stale(
            cr, 
            uid, 
            ('in', 'internal'), 
            'Zaber', 
            'po_stale', 
            context=context)
        self.po_stale(
            cr, 
            uid, 
            ('out',), 
            'customers', 
            'po_stale_out',
            include_destination_xml_id='stock_location_customers', 
            context=context)
        self.po_stale(
            cr, 
            uid, 
            ('out',), 
            'suppliers', 
            'po_stale_to_supp', 
            exclude_destination_xml_id='stock_location_customers', 
            context=context)
        return 0
    
    def __days_ago(self, days):
        " Calculate a timestamp from so many days ago. Returned as a string."
        
        now = datetime.datetime.now()
        timestamp = now - datetime.timedelta(days)
        return timestamp.strftime('%Y-%m-%d')

    def po_stale(self, 
                 cr, 
                 uid, 
                 types, 
                 description, 
                 case_code, 
                 include_destination_xml_id=None, 
                 exclude_destination_xml_id=None, 
                 context=None):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(
            cr,
            uid,
            context,
            "Found active packings to %s with old dates" % description,
            "List the identified packing orders in a table with columns for " +
                "type (incoming, internal, or outgoing), status, ref " +
                "(e.g., PACK4471), creator, planned date.",
            case_code)
        
        min_confirmed_date = self.__days_ago(30)
        min_other_date = self.__days_ago(1)

        picking_ids = self.search(cr,uid,[
            ('type', 'in', types),
            '|',
            '&',
            ('state', '=', 'confirmed'),
            ('min_date', '<=', min_confirmed_date),
            '&',
            ('state', 'in', ['draft', 'auto', 'assigned']),
            ('min_date', '<=', min_other_date),
        ])

        all_pickings = self.browse(cr,uid,picking_ids)
        if not (include_destination_xml_id or exclude_destination_xml_id):
            pickings = all_pickings
        else:
            xml_id = include_destination_xml_id or exclude_destination_xml_id
            data_obj = self.pool.get('ir.model.data')
            data_id = data_obj._get_id(cr, uid, 'stock', xml_id)
            destination_id = data_obj.browse(cr, uid, data_id).res_id

            pickings = []
            for picking in self.browse(cr, uid, picking_ids):
                for move in picking.move_lines:
                    to_destination = move.location_dest_id.id == destination_id
                    if to_destination ^ bool(exclude_destination_xml_id):
                        pickings.append(picking)
                        break

        if not pickings:
            return

        reports = []

        state_map = dict(self._columns['state'].selection)
        type_map = dict(self._columns['type'].selection)
        for picking in pickings:
            template = (" - {state:<15}\t{picking.min_date:<15}\t{type:<15}\t" +
                        "{picking.name}")
            description = template.format(
                picking=picking,
                type=type_map[picking.type],
                state=state_map[picking.state])
            reports.append((picking.state, picking.min_date, description))

        reports.sort()
        problem_writeup = '\n'.join([
            report[2] 
            for report in reports 
            if report[0] == 'confirmed']) 
        if problem_writeup:
            problem_writeup += '\n\n'
        problem_writeup += '\n'.join([
            report[2] 
            for report in reports 
            if report[0] != 'confirmed'])
        problem_writeup = (
            "These packing lists have an old planned date:\n\n" +
            problem_writeup)
        
        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(
            cr, uid, context,
            case_id,
            problem_writeup)

stock_picking()
